@@ -20,6 +20,7 @@ import os
 import socket
 import StringIO
 import sys
+import time
 import tracing
 import ttystatus
 
@@ -75,7 +76,13 @@ class App(cliapp.Application):
 
         self.settings.boolean(['pretend', 'dry-run', 'no-act'],
                            'do not actually change anything (works with '
-                           'forget and restore only)')
+                           'backup, forget and restore only, and may only '
+                           'simulate approximately real behavior)')
+                           
+        self.settings.string(['pretend-time'],
+                             'pretend it is TIMESTAMP (YYYY-MM-DD HH:MM:SS); '
+                                'this is only useful for testing purposes',
+                             metavar='TIMESTAMP')
 
         # The following needs to be done here, because it needs
         # to be done before option processing. This is a bit ugly,
@@ -111,11 +118,19 @@ class App(cliapp.Application):
         # and since all hooks must be defined when plugins are enabled,
         # we create one instance here, which will immediately be destroyed.
         # FIXME: This is fugly.
-        obnamlib.Repository(None, 1000, 1000, 100, self.hooks, 10, 10, 10)
+        obnamlib.Repository(None, 1000, 1000, 100, self.hooks, 10, 10, 10,
+                            self.time)
 
     def plugins_dir(self):
         return os.path.join(os.path.dirname(obnamlib.__file__), 'plugins')
 
+    def setup_logging(self):
+        log = self.settings['log']
+        if log and log != 'syslog' and not os.path.exists(log):
+            fd = os.open(log, os.O_WRONLY | os.O_CREAT, 0600)
+            os.close(fd)
+        cliapp.Application.setup_logging(self)
+
     def process_args(self, args):
         if self.settings['quiet']:
             self.ts.disable()
@@ -151,5 +166,21 @@ class App(cliapp.Application):
                                     self.hooks,
                                     self.settings['idpath-depth'],
                                     self.settings['idpath-bits'],
-                                    self.settings['idpath-skip'])
+                                    self.settings['idpath-skip'],
+                                    self.time)
+
+    def time(self):
+        '''Return current time in seconds since epoch.
+        
+        This is a wrapper around time.time() so that it can be overridden
+        with the --pretend-time setting.
+        
+        '''
+
+        s = self.settings['pretend-time']
+        if s:
+            t = time.strptime(s, '%Y-%m-%d %H:%M:%S')
+            return time.mktime(t)
+        else:
+            return time.time()
 
