@@ -87,6 +87,7 @@ class SSHChannelAdapter(object):
         return 'obnam SSHChannelAdapter'
 
     def close(self):
+        logging.debug('SSHChannelAdapter.close called')
         for func in [self.proc.stdin.close, self.proc.stdout.close, 
                      self.proc.wait]:
             try:
@@ -126,27 +127,31 @@ class SftpFS(obnamlib.VirtualFileSystem):
             return str_or_unicode
         
     def connect(self):
-        if not self._connect_openssh():
+        try_openssh = not self.settings or not self.settings['pure-paramiko']
+        if not try_openssh or not self._connect_openssh():
             self._connect_paramiko()
         if self.create_path_if_missing and not self.exists(self.path):
             self.mkdir(self.path)
         self.chdir(self.path)
 
-    def _connect_paramiko(self):
-        self.transport = paramiko.Transport((self.host, self.port))
-        self.transport.connect()
-        self._check_host_key(self.host)
-        self._authenticate(self.user)
-        self.sftp = paramiko.SFTPClient.from_transport(self.transport)
-
     def _connect_openssh(self):
         args = ['ssh',
                 '-oForwardX11=no', '-oForwardAgent=no',
                 '-oClearAllForwardings=yes', '-oProtocol=2',
                 '-p', str(self.port),
                 '-l', self.user,
-                '-s', self.host, 'sftp']
+                '-s']
+        if self.settings and self.settings['ssh-key']:
+            args += ['-i', self.settings['ssh-key']]
+        if self.settings and self.settings['strict-ssh-host-keys']:
+            args += ['-o', 'StrictHostKeyChecking=yes']
+        if self.settings and self.settings['ssh-known-hosts']:
+            args += ['-o', 
+                     'UserKnownHostsFile=%s' % 
+                        self.settings['ssh-known-hosts']]
+        args += [self.host, 'sftp']
 
+        logging.debug('executing openssh: %s' % args)
         try:
             proc = subprocess.Popen(args,
                                     stdin=subprocess.PIPE,
@@ -159,34 +164,89 @@ class SftpFS(obnamlib.VirtualFileSystem):
         self.sftp = paramiko.SFTPClient(SSHChannelAdapter(proc))
         return True
 
+    def _connect_paramiko(self):
+        logging.debug('connect_paramiko: host=%s port=%s' % (self.host, self.port))
+        self.transport = paramiko.Transport((self.host, self.port))
+        self.transport.connect()
+        logging.debug('connect_paramiko: connected')
+        try:
+            self._check_host_key(self.host)
+        except BaseException, e:
+            self.transport.close()
+            self.transport = None
+            raise
+        logging.debug('connect_paramiko: host key checked')
+        self._authenticate(self.user)
+        logging.debug('connect_paramiko: authenticated')
+        self.sftp = paramiko.SFTPClient.from_transport(self.transport)
+        logging.debug('connect_paramiko: end')
+
     def _check_host_key(self, hostname):
-        key = self.transport.get_remote_server_key()
-        known_hosts = os.path.expanduser('~/.ssh/known_hosts')
-        keys = paramiko.util.load_host_keys(known_hosts)
-        if hostname not in keys:
-            raise obnamlib.Error('Host not in known_hosts: %s' % hostname)
-        elif not keys[hostname].has_key(key.get_name()):
-            raise obnamlib.Error('No host key for %s' % hostname)
-        elif keys[hostname][key.get_name()] != key:
-            raise obnamlib.Error('Host key has changed for %s' % hostname)
+        logging.debug('checking ssh host key for %s' % hostname)
+
+        offered_key = self.transport.get_remote_server_key()
+
+        known_hosts_path = self.settings['ssh-known-hosts']
+        known_hosts = paramiko.util.load_host_keys(known_hosts_path)
+
+        known_keys = known_hosts.lookup(hostname)
+        if known_keys is None:
+            if self.settings['strict-ssh-host-keys']:
+                raise obnamlib.Error('No known host key for %s' % hostname)
+            logging.warning('No known host keys for %s; accepting offered key'
+                            % hostname)
+            return
+
+        offered_type = offered_key.get_name()
+        if not known_keys.has_key(offered_type):
+            if self.settings['strict-ssh-host-keys']:
+                raise obnamlib.Error('No known type %s host key for %s' % 
+                                     (offered_type, hostname))
+            logging.warning('No known host key of type %s for %s; accepting '
+                            'offered key' % (offered_type, hostname))
+        
+        known_key = known_keys[offered_type]
+        if offered_key != known_key:
+            raise obnamlib.Error('SSH server %s offered wrong public key' %
+                                 hostname)
+            
+        logging.debug('Host key for %s OK' % hostname)        
     
     def _authenticate(self, username):
-        agent = paramiko.Agent()
-        agent_keys = agent.get_keys()
-        for key in agent_keys:
+        for key in self._find_auth_keys():
             try:
                 self.transport.auth_publickey(username, key)
                 return
             except paramiko.SSHException:
                 pass
-        raise obnamlib.Error('Can\'t authenticate to SSH server using agent.')
+        raise obnamlib.Error('Can\'t authenticate to SSH server using key.')
+
+    def _find_auth_keys(self):
+        if self.settings and self.settings['ssh-key']:
+            return [self._load_from_key_file(self.settings['ssh-key'])]
+        else:
+            return self._load_from_agent()
+
+    def _load_from_key_file(self, filename):
+        try:
+            key = paramiko.RSAKey.from_private_key_file(filename)
+        except paramiko.PasswordRequiredException:
+            password = getpass.getpass('RSA key password for %s: ' %
+                                        filename)
+            key = paramiko.RSAKey.from_private_key_file(filename, password)
+        return key
+
+    def _load_from_agent(self):
+        agent = paramiko.Agent()
+        return agent.get_keys()
 
     def close(self):
+        logging.debug('SftpFS.close called')
         self.sftp.close()
+        self.sftp = None
         if self.transport:
             self.transport.close()
             self.transport = None
-        self.sftp = None
         obnamlib.VirtualFileSystem.close(self)
         self._delay()
 
@@ -491,5 +551,28 @@ class SftpPlugin(obnamlib.ObnamPlugin):
         self.app.settings.integer(['sftp-delay'],
                                   'add an artificial delay (in milliseconds) '
                                     'to all SFTP transfers')
+
+        self.app.settings.string(['ssh-key'],
+                                 'use FILENAME as the ssh RSA private key for '
+                                    'sftp access (default is using keys known '
+                                    'to ssh-agent)',
+                                 metavar='FILENAME')
+
+        self.app.settings.boolean(['strict-ssh-host-keys'],
+                                  'require that the ssh host key must be '
+                                    'known and correct to be accepted; '
+                                    'default is to accept unknown keys')
+
+        self.app.settings.string(['ssh-known-hosts'],
+                                 'filename of the user\'s known hosts file '
+                                    '(default: %default)',
+                                 metavar='FILENAME',
+                                 default=
+                                    os.path.expanduser('~/.ssh/known_hosts'))
+
+        self.app.settings.boolean(['pure-paramiko'],
+                                 'do not use openssh even if available, '
+                                    'use paramiko only instead')
+
         self.app.fsf.register('sftp', SftpFS, settings=self.app.settings)
 
