@@ -19,6 +19,7 @@ import os
 import shutil
 import stat
 import tempfile
+import time
 import unittest
 
 import obnamlib
@@ -35,7 +36,8 @@ class RepositoryRootNodeTests(unittest.TestCase):
                                         obnamlib.DEFAULT_LRU_SIZE, None,
                                         obnamlib.IDPATH_DEPTH,
                                         obnamlib.IDPATH_BITS,
-                                        obnamlib.IDPATH_SKIP)
+                                        obnamlib.IDPATH_SKIP,
+                                        time.time)
         
         self.otherfs = obnamlib.LocalFS(self.tempdir)
         self.other = obnamlib.Repository(self.fs, obnamlib.DEFAULT_NODE_SIZE,
@@ -43,7 +45,8 @@ class RepositoryRootNodeTests(unittest.TestCase):
                                          obnamlib.DEFAULT_LRU_SIZE, None,
                                         obnamlib.IDPATH_DEPTH,
                                         obnamlib.IDPATH_BITS,
-                                        obnamlib.IDPATH_SKIP)
+                                        obnamlib.IDPATH_SKIP,
+                                        time.time)
 
     def tearDown(self):
         shutil.rmtree(self.tempdir)
@@ -158,7 +161,8 @@ class RepositoryRootNodeTests(unittest.TestCase):
                                  obnamlib.DEFAULT_LRU_SIZE, None,
                                  obnamlib.IDPATH_DEPTH,
                                  obnamlib.IDPATH_BITS,
-                                 obnamlib.IDPATH_SKIP)
+                                 obnamlib.IDPATH_SKIP,
+                                 time.time)
         self.assertEqual(s2.list_clients(), ['foo'])
         
     def test_adding_existing_client_fails(self):
@@ -227,7 +231,8 @@ class RepositoryClientTests(unittest.TestCase):
                                         obnamlib.DEFAULT_LRU_SIZE, None,
                                         obnamlib.IDPATH_DEPTH,
                                         obnamlib.IDPATH_BITS,
-                                        obnamlib.IDPATH_SKIP)
+                                        obnamlib.IDPATH_SKIP,
+                                        time.time)
         self.repo.lock_root()
         self.repo.add_client('client_name')
         self.repo.commit_root()
@@ -239,7 +244,8 @@ class RepositoryClientTests(unittest.TestCase):
                                          obnamlib.DEFAULT_LRU_SIZE, None,
                                          obnamlib.IDPATH_DEPTH,
                                          obnamlib.IDPATH_BITS,
-                                         obnamlib.IDPATH_SKIP)
+                                         obnamlib.IDPATH_SKIP,
+                                         time.time)
         
         self.dir_meta = obnamlib.Metadata()
         self.dir_meta.st_mode = stat.S_IFDIR | 0777
@@ -517,7 +523,8 @@ class RepositoryChunkTests(unittest.TestCase):
                                         obnamlib.DEFAULT_LRU_SIZE, None,
                                         obnamlib.IDPATH_DEPTH,
                                         obnamlib.IDPATH_BITS,
-                                        obnamlib.IDPATH_SKIP)
+                                        obnamlib.IDPATH_SKIP,
+                                        time.time)
         self.repo.lock_root()
         self.repo.add_client('client_name')
         self.repo.commit_root()
@@ -589,7 +596,8 @@ class RepositoryGetSetChunksTests(unittest.TestCase):
                                         obnamlib.DEFAULT_LRU_SIZE, None,
                                         obnamlib.IDPATH_DEPTH,
                                         obnamlib.IDPATH_BITS,
-                                        obnamlib.IDPATH_SKIP)
+                                        obnamlib.IDPATH_SKIP,
+                                        time.time)
         self.repo.lock_root()
         self.repo.add_client('client_name')
         self.repo.commit_root()
@@ -633,7 +641,8 @@ class RepositoryGenspecTests(unittest.TestCase):
                                         obnamlib.DEFAULT_LRU_SIZE, None,
                                         obnamlib.IDPATH_DEPTH,
                                         obnamlib.IDPATH_BITS,
-                                        obnamlib.IDPATH_SKIP)
+                                        obnamlib.IDPATH_SKIP,
+                                        time.time)
         self.repo.lock_root()
         self.repo.add_client('client_name')
         self.repo.commit_root()
@@ -685,7 +694,8 @@ class RepositoryWalkTests(unittest.TestCase):
                                         obnamlib.DEFAULT_LRU_SIZE, None,
                                         obnamlib.IDPATH_DEPTH,
                                         obnamlib.IDPATH_BITS,
-                                        obnamlib.IDPATH_SKIP)
+                                        obnamlib.IDPATH_SKIP,
+                                        time.time)
         self.repo.lock_root()
         self.repo.add_client('client_name')
         self.repo.commit_root()
